==> fhcore-libs-0.2.5.4.tar.gz: fhcore-libs-0.2.5.4/fhcore/models/base.py <==
# -*- coding: utf-8 -*-
""" FHCoreLibs > MODELS > Base.py by Frankhood
    @author: Frankhood Business Solutions
"""
import time
import datetime
import itertools

from django.conf import settings
from django.contrib.auth.models import User, SiteProfileNotAvailable
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.generic import GenericRelation
from django.contrib.sites.models import Site
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _



from fhcore.managers import CurrentSiteEntryManager, NotPreviewCategoryManager, NotPreviewEntryManager, NotPreviewPublishedCategoryManager,\
                     NotPreviewPublishedEntryManager, PublishedCategoryManager, PublishedEntryManager
from fhcore.models.mixins import PublicableModelMixin, LocaleModelMixin, OrderableModelMixin

AVAILABLE_CONTENT_TYPES = models.Q(app_label__in = settings.PROJECT_APPS)


class BaseModel(models.Model):
    """ classe model astratta dalla quale tutti i models concreti dovrebbero ereditare """
    
    created = models.DateTimeField(_("created at"), auto_now_add=True)
    modified = models.DateTimeField(_("modified at"), auto_now=True)
    created_by = models.CharField(_('created by'), max_length=155 ,blank=True, editable=False)
    modified_by = models.CharField(_('modified by'), max_length=155, blank=True, editable=False)
    
    class Meta:
        abstract = True
        ordering = ['-created']
        get_latest_by = '-created'
       
class BaseCategory(BaseModel):

    name = models.CharField(_("name"), max_length=155)
    slug = models.SlugField()
    sort_num = models.IntegerField(_("order"), blank=True)
    css_class = models.CharField(_("CSS class"),  max_length=32, blank=True)

    class Meta(BaseModel.Meta):
        abstract = True
        #unique_together = ("slug", ) 
        ordering = ['sort_num', '-created']
        
    def _get_ordering_queryset(self):
        return self.__class__._default_manager.all()
    
    def get_previous(self, queryset=None):
        if queryset is None:
            queryset = self._get_ordering_queryset()
        return queryset.order_by('-sort_num', '-id').filter(sort_num__lt=self.sort_num)[0:1].get()

    def get_next(self, queryset=None):
        if queryset is None:
            queryset = self._get_ordering_queryset()
        return queryset.order_by('sort_num', 'id').filter(sort_num__gt=self.sort_num)[0:1].get()
        
    def save(self, *args, **kwargs):
        if not self.slug:
            slug = slugify(self.name)[:40]
            try:
                self.__class__._default_manager.get(slug__iexact=slug)
            except self.__class__.DoesNotExist:
                self.slug = slug
            else:
                #NOTE: non posso basarmi sull'id poichè non è stato ancora generato.
                n = datetime.datetime.now()
                t = int(time.mktime(n.timetuple()))
                self.slug = '%s-%d' % (slug, t)
                self.slug = self.slug[:50]
        else:
            slug = slugify(self.name)[:40]
            try:
                obj = self.__class__._default_manager.get(slug=self.slug)
                if obj != self:
                    if self.id: 
                        self.slug = '%s-%d' % (self.slug, self.id)
                    else: 
                        n = datetime.datetime.now()
                        t = int(time.mktime(n.timetuple()))
                        self.slug = '%s-%d' % (slug, t)
                        self.slug = self.slug[:50]
                        #self.slug = self.slug[:-1]
            except self.__class__.DoesNotExist:
                pass
            except MultipleObjectsReturned, ex:
                pass  
        queryset = self._get_ordering_queryset()
        if self.sort_num is None:
            max = queryset.aggregate(models.Max('sort_num'))
            try:
                self.sort_num = max.get('sort_num__max', 0) + 1
            except TypeError:
                self.sort_num = 0
        if not self.id:
            qs = queryset.filter(sort_num__gte=self.sort_num).update(sort_num=models.F('sort_num')+1)
        else:
            try:
                old_obj = queryset.get(id=self.id)
                if old_obj.sort_num is not None:
                    if old_obj.sort_num > self.sort_num:
                        qs = queryset.filter(models.Q(sort_num__gte=self.sort_num),models.Q(sort_num__lt=old_obj.sort_num)).update(sort_num=models.F('sort_num')+1)
                    elif old_obj.sort_num < self.sort_num:
                        qs = queryset.filter(models.Q(sort_num__lte=self.sort_num),models.Q(sort_num__gt=old_obj.sort_num)).update(sort_num=models.F('sort_num')-1)
            except (ObjectDoesNotExist, MultipleObjectsReturned), ex:
                pass
        return super(BaseCategory, self).save(*args, **kwargs)
     
    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "name__icontains",)   
    
class BaseLocaleCategory(LocaleModelMixin, BaseCategory):
    
    class Meta(BaseCategory.Meta):
        abstract = True
        unique_together = ("slug", "lang")

class AdvancedCategory(PublicableModelMixin, LocaleModelMixin, BaseCategory):
    
    objects = models.Manager()
    published_objects = PublishedCategoryManager()
    
    class Meta(BaseCategory.Meta):
        abstract = True
     

class BaseEntryModel(BaseModel):
    
    #on_site = CurrentSiteEntryManager(field_name='sites')
    #sites = models.ManyToManyField(Site, verbose_name=_("published on"))
    
    title = models.CharField(_("title"), max_length=155)
    slug = models.SlugField() #eliminato unique=True x la questione della preview e del linguaggio... 
    subtitle = models.CharField(_("subtitle"), max_length=155, blank=True)
    content = models.TextField(_("content"), blank=True)
    excerpt = models.TextField(_("abstract"), blank=True)
    

    class Meta(BaseModel.Meta):
        abstract = True
        #unique_together = ("slug", ) 
        
    def __unicode__(self):
        return self.title
    
    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "title__icontains",)   

    
    '''
    def save(self, *args, **kwargs):
        if not self.slug:
            slug = slugify(self.title)[:50]
            try:
                self.__class__._default_manager.get(slug__iexact=slug)
            except self.__class__.DoesNotExist:
                self.slug = slug
            else:
                #NOTE: non posso basarmi sull'id poichè non è stato ancora generato.
                n = datetime.datetime.now()
                t = int(time.mktime(n.timetuple()))
                self.slug = '%s-%d' % (slug, t)
        else:
            try:
                obj = self.__class__.objects.get(slug=self.slug)
                if obj != self:
                    if self.id: self.slug = '%s-%d' % (self.slug, self.id)
                    else: self.slug = self.slug[:-1]
            except self.__class__.DoesNotExist:
                pass
        super(BaseEntryModel, self).save(*args, **kwargs)    
    '''

class PublicableEntryModel(PublicableModelMixin, BaseEntryModel):
    
    objects = models.Manager() 
    published_objects = PublishedEntryManager()
    
    class Meta(PublicableModelMixin.Meta, BaseEntryModel.Meta):
        abstract = True
        ordering = ['-publication_start', '-created']

class AdvancedLocaleModel(OrderableModelMixin, LocaleModelMixin, PublicableEntryModel):
    
    sites = models.ManyToManyField(Site, verbose_name=_("published on"), blank=True, null=True)
    
    class Meta(OrderableModelMixin.Meta, PublicableEntryModel.Meta):
        abstract = True
        ordering = ['order', '-publication_start', '-created'] 

class AdvancedEntryModel(PublicableEntryModel):
    
    objects = models.Manager() 
    published_objects = PublishedEntryManager()
    
    author = models.ForeignKey(User, verbose_name=_("author"), related_name='%(class)s_written', null=True, blank=True)
    #tags = TagField(_('tags'), blank=True)
    
    # generic relatioship for related objects
    content_type = models.ForeignKey(ContentType,limit_choices_to=AVAILABLE_CONTENT_TYPES,  null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    related_objects = GenericForeignKey()
    
    class Meta(PublicableEntryModel.Meta):
        abstract = True
        ordering = ['-publication_start', '-created']
        #unique_together = ("slug", "is_preview") 

    def __unicode__(self):
        return '%s, %s' %(self.title, self.author)
        
    """
    @property
    def tag_list(self):
        ''' tag list
            @return: list
        '''
        return parse_tag_input(self.tags)
    
    @property
    def has_tags(self):
        return bool(len(self.tag_list))
    
    @property
    def keywords(self):
        ''' keyword list to feed a metatag 
            @return: list
        '''
        kw_data = set(self.title.split() + self.tag_list)
        return list(kw_data)
    
    def get_related(self, related_models=None, limit=3):
        if not related_models:
            related_models = [self.__class__]
        related_list = []
        for model in related_models:
            qs = TaggedItem.objects.get_related(self, model.published_objects.all(), limit)
            related_list.append(qs)
        return list(itertools.chain(*related_list))
    """ 


    
class AdvancedItemModel(LocaleModelMixin, OrderableModelMixin, AdvancedEntryModel):
    
    class Meta(AdvancedEntryModel.Meta):
        abstract = True
        #unique_together = ("slug", "lang")
    

class AdvancedArticleModel(LocaleModelMixin, AdvancedEntryModel):
    
    class Meta(AdvancedEntryModel.Meta):
        abstract = True
        unique_together = ("slug", "lang")

==> fhcore-libs-0.2.6.2.tar.gz: fhcore-libs-0.2.6.2/fhcore/models/base.py <==
# -*- coding: utf-8 -*-
""" FHCoreLibs > MODELS > Base.py by Frankhood
    @author: Frankhood Business Solutions
"""
import time
import datetime
import itertools

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.generic import GenericRelation
from django.contrib.sites.models import Site
from django.core.exceptions import MultipleObjectsReturned, ObjectDoesNotExist
from django.db import models
from django.template.defaultfilters import slugify
from django.utils.translation import ugettext_lazy as _

from fhcore.managers import CurrentSiteEntryManager, NotPreviewCategoryManager, NotPreviewEntryManager, NotPreviewPublishedCategoryManager,\
                     NotPreviewPublishedEntryManager, PublishedCategoryManager, PublishedEntryManager
from fhcore.models.mixins import PublicableModelMixin, LocaleModelMixin, OrderableModelMixin, SlugModelMixin

AVAILABLE_CONTENT_TYPES = models.Q(app_label__in = settings.PROJECT_APPS)


class BaseModel(models.Model):
    """ classe model astratta dalla quale tutti i models concreti dovrebbero ereditare """
    
    created = models.DateTimeField(_("created at"), auto_now_add=True)
    modified = models.DateTimeField(_("modified at"), auto_now=True)
    created_by = models.CharField(_('created by'), max_length=155 ,blank=True, editable=False)
    modified_by = models.CharField(_('modified by'), max_length=155, blank=True, editable=False)
    
    class Meta:
        abstract = True
        ordering = ['-created']
        get_latest_by = '-created'

    @property
    def absolute_url(self):
        from django.core.urlresolvers import reverse
        url_context = {'app_label':self._meta.app_label,'model_name':self._meta.model_name}
        return reverse('admin:{app_label}_{model_name}_change'.format(**url_context), 
                       args=(self.id,))
       
class BaseCategory(SlugModelMixin, BaseModel):
    slugged_field = 'name' 
    
    name = models.CharField(_("name"), max_length=155)
    #slug = models.SlugField() #eliminato unique=True x la questione della preview e del linguaggio... 
    sort_num = models.IntegerField(_("order"), blank=True, default=0)
    css_class = models.CharField(_("CSS class"),  max_length=32, blank=True)

    class Meta(BaseModel.Meta):
        abstract = True
        #unique_together = ("slug", ) 
        ordering = ['sort_num', '-created']
        
    def _get_ordering_queryset(self):
        return self.__class__._default_manager.all()
    
    def get_previous(self, queryset=None):
        if queryset is None:
            queryset = self._get_ordering_queryset()
        return queryset.order_by('-sort_num', '-id').filter(sort_num__lt=self.sort_num)[0:1].get()

    def get_next(self, queryset=None):
        if queryset is None:
            queryset = self._get_ordering_queryset()
        return queryset.order_by('sort_num', 'id').filter(sort_num__gt=self.sort_num)[0:1].get()
    
    """    
    def save(self, *args, **kwargs):
        if not self.slug:
            slug = slugify(self.name)[:40]
            try:
                self.__class__._default_manager.get(slug__iexact=slug)
            except self.__class__.DoesNotExist:
                self.slug = slug
            else:
                #NOTE: non posso basarmi sull'id poichè non è stato ancora generato.
                n = datetime.datetime.now()
                t = int(time.mktime(n.timetuple()))
                self.slug = '%s-%d' % (slug, t)
                self.slug = self.slug[:50]
        else:
            slug = slugify(self.name)[:40]
            try:
                obj = self.__class__._default_manager.get(slug=self.slug)
                if obj != self:
                    if self.id: 
                        self.slug = '%s-%d' % (self.slug, self.id)
                    else: 
                        n = datetime.datetime.now()
                        t = int(time.mktime(n.timetuple()))
                        self.slug = '%s-%d' % (slug, t)
                        self.slug = self.slug[:50]
                        #self.slug = self.slug[:-1]
            except self.__class__.DoesNotExist:
                pass
            except MultipleObjectsReturned, ex:
                pass  
        queryset = self._get_ordering_queryset()
        if self.sort_num is None:
            max = queryset.aggregate(models.Max('sort_num'))
            try:
                self.sort_num = max.get('sort_num__max', 0) + 1
            except TypeError:
                self.sort_num = 0
        if not self.id:
            qs = queryset.filter(sort_num__gte=self.sort_num).update(sort_num=models.F('sort_num')+1)
        else:
            try:
                old_obj = queryset.get(id=self.id)
                if old_obj.sort_num is not None:
                    if old_obj.sort_num > self.sort_num:
                        qs = queryset.filter(models.Q(sort_num__gte=self.sort_num),models.Q(sort_num__lt=old_obj.sort_num)).update(sort_num=models.F('sort_num')+1)
                    elif old_obj.sort_num < self.sort_num:
                        qs = queryset.filter(models.Q(sort_num__lte=self.sort_num),models.Q(sort_num__gt=old_obj.sort_num)).update(sort_num=models.F('sort_num')-1)
            except (ObjectDoesNotExist, MultipleObjectsReturned), ex:
                pass
        return super(BaseCategory, self).save(*args, **kwargs)
    """
     
    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "name__icontains",)   
    
class BaseLocaleCategory(LocaleModelMixin, BaseCategory):
    
    class Meta(BaseCategory.Meta):
        abstract = True
        unique_together = ("slug", "lang")

class AdvancedCategory(PublicableModelMixin, LocaleModelMixin, BaseCategory):
    
    objects = models.Manager()
    published_objects = PublishedCategoryManager()
    
    class Meta(BaseCategory.Meta):
        abstract = True
     

class BaseEntryModel(SlugModelMixin, BaseModel):
    slugged_field = 'title' 
    
    #on_site = CurrentSiteEntryManager(field_name='sites')
    #sites = models.ManyToManyField(Site, verbose_name=_("published on"))
    
    title = models.CharField(_("title"), max_length=155)
    #slug = models.SlugField() #eliminato unique=True x la questione della preview e del linguaggio... 
    subtitle = models.CharField(_("subtitle"), max_length=155, blank=True)
    content = models.TextField(_("content"), blank=True)
    excerpt = models.TextField(_("abstract"), blank=True)
    

    class Meta(BaseModel.Meta):
        abstract = True
        #unique_together = ("slug", ) 
        
    def __unicode__(self):
        return self.title
    
    @staticmethod
    def autocomplete_search_fields():
        return ("id__iexact", "title__icontains",)   
    

class PublicableEntryModel(PublicableModelMixin, BaseEntryModel):
    
    objects = models.Manager() 
    published_objects = PublishedEntryManager()
    
    class Meta(PublicableModelMixin.Meta, BaseEntryModel.Meta):
        abstract = True
        ordering = ['-publication_start', '-created']

class AdvancedLocaleModel(OrderableModelMixin, LocaleModelMixin, PublicableEntryModel):
    
    sites = models.ManyToManyField(Site, verbose_name=_("published on"), blank=True, null=True)
    
    class Meta(OrderableModelMixin.Meta, PublicableEntryModel.Meta):
        abstract = True
        ordering = ['order', '-publication_start', '-created'] 

class AdvancedEntryModel(PublicableEntryModel):
    
    objects = models.Manager() 
    published_objects = PublishedEntryManager()
    
    author = models.ForeignKey(User, verbose_name=_("author"), related_name='%(class)s_written', null=True, blank=True)
    #tags = TagField(_('tags'), blank=True)
    
    # generic relatioship for related objects
    content_type = models.ForeignKey(ContentType,limit_choices_to=AVAILABLE_CONTENT_TYPES,  null=True, blank=True)
    object_id = models.PositiveIntegerField(null=True, blank=True)
    related_objects = GenericForeignKey()
    
    class Meta(PublicableEntryModel.Meta):
        abstract = True
        ordering = ['-publication_start', '-created']
        #unique_together = ("slug", "is_preview") 

    def __unicode__(self):
        return '%s, %s' %(self.title, self.author)
        
    """
    @property
    def tag_list(self):
        ''' tag list
            @return: list
        '''
        return parse_tag_input(self.tags)
    
    @property
    def has_tags(self):
        return bool(len(self.tag_list))
    
    @property
    def keywords(self):
        ''' keyword list to feed a metatag 
            @return: list
        '''
        kw_data = set(self.title.split() + self.tag_list)
        return list(kw_data)
    
    def get_related(self, related_models=None, limit=3):
        if not related_models:
            related_models = [self.__class__]
        related_list = []
        for model in related_models:
            qs = TaggedItem.objects.get_related(self, model.published_objects.all(), limit)
            related_list.append(qs)
        return list(itertools.chain(*related_list))
    """ 


    
class AdvancedItemModel(LocaleModelMixin, OrderableModelMixin, AdvancedEntryModel):
    
    class Meta(AdvancedEntryModel.Meta):
        abstract = True
        #unique_together = ("slug", "lang")
    

class AdvancedArticleModel(LocaleModelMixin, AdvancedEntryModel):
    
    class Meta(AdvancedEntryModel.Meta):
        abstract = True
        unique_together = ("slug", "lang")
